
import sys
import time

from AM_CommonTools.util.time_helper import TimeHelper
from TangentV.indexing.recognition_helper import RecognitionHelper

from recognizer.configuration.configuration import Configuration


def main():
    # validate params
    if len(sys.argv) < 3:
        print("Usage:")
        print("")
        print("python {0:s} conf_file input_dir".format(sys.argv[0]))
        print("")
        print("Where")
        print("\tconf_file:\tFile with symbol recognizer configuration")
        print("\tinput_dir:\tInput Directory where rendered traces were stored ")
        return

    config_filename = sys.argv[1]
    input_dir = sys.argv[2]

    total_time_loading = 0.0
    total_time_feat_extract = 0.0
    total_time_storing = 0.0

    start_loading = time.time()

    # load configuration
    config = Configuration.from_file(config_filename)

    all_datasets = [
        ("training", config.data["TRAINING_DATASET_NAME"] + ".dat"),
        ("testing", config.data["TESTING_DATASET_NAME"] + ".dat")
    ]

    # load class mapping ...
    mapping_filename = input_dir + "/class_mapping.txt"
    label_mapping, class_mapping = RecognitionHelper.load_label_mapping(mapping_filename)

    workers = config.get_int("PARALLEL_INKML_LOADING_WORKERS", 10)

    normalizer = None

    # for training / testing
    for input_prefix, output_suffix in all_datasets:
        # get sub-directories
        current_directory = input_dir + "/" + input_prefix

        task_start_time = time.time()
        dataset = RecognitionHelper.extract_features_from_img_dirs(current_directory, label_mapping, config, normalizer,
                                                                   workers, True)

        if normalizer is None:
            # keep only the training set normalizer ....
            normalizer = dataset.normalizer

        task_end_time = time.time()
        total_time_feat_extract += task_end_time - task_start_time
        print("... Feature extraction complete!")

        print("Saving ... ")

        # ... save to file ....
        task_start_time = time.time()
        dataset.save_to_file(config.data["DATASET_DIR"] + "/" + output_suffix)
        task_end_time = time.time()
        total_time_storing += task_end_time - task_start_time

        # releasing memory ...
        dataset = None

    end_time = time.time()

    print("Total Time Loading: " + TimeHelper.secondsToStr(total_time_loading))
    print("Total Time for Feature Extraction: " + TimeHelper.secondsToStr(total_time_feat_extract))
    print("Total Time saving: " + TimeHelper.secondsToStr(total_time_storing))
    print("Overall Time: " + TimeHelper.secondsToStr(end_time - start_loading))


if __name__ == "__main__":
    main()

